-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

-- Prep the module
local M = {}
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 11000

-- Debug stuff
local ugfxCalled = 0

-- Fire nodes
local nFtop = nil
local nFbtm = nil
local canFire = 0

-- Last button states
local lastBurnOff = 0
local lastBurnSmolder = 0
local lastBurnFire = 0
local lastBurnExplosion = 0

-- Burn states
local burnState = 0
local burningNot = 0
local burningSmolder = 1
local burningFire = 2
local burningExplosion = 3
local burningNoChange = 4

-- Particle emission timing
local burnTime = 0
local burnInterval = 0.0333

-- Explosion timing
local expTimeElap = 0
local explosionDuration = 0.5

-- Smoke and fire metrics
local smolderSmokeSpeed = 1
local smolderSmokeWidth = 0.1
local smolderSmokeCount = 1
local fireFlameSpeed = 0
local fireFlameWidth = 0.1
local fireFlameCount = 1
local fireSmokeThick = 43 -- or 52
local fireSmokeSpeed = 10
local fireSmokeWidth = 1.5
local fireSmokeCount = 1
local explosionFlameSpeed = 0
local explosionFlameWidth = 5
local explosionFlameCount = 20
local explosionSmokeSpeed = 10
local explosionSmokeWidth = 5
local explosionSmokeCount = 2

-- Light metrics
local castLight = 1

-- Determine dominant button state
local function nextButtonBurnState()
	
	-- Default to no changes
	local nextState = burningNoChange
	
	-- Get current button states
	local thisBurnOff = (electrics.values['burnOff'] or 0)
	local thisBurnSmolder = (electrics.values['burnSmolder'] or 0)
	local thisBurnFire = (electrics.values['burnFire'] or 0)
	local thisBurnExplosion = (electrics.values['burnExplosion'] or 0)
	
	-- Check for burn button state changes
	if lastBurnExplosion == 0 and thisBurnExplosion == 1 then nextState = burningExplosion
	elseif lastBurnFire == 0 and thisBurnFire == 1 then       nextState = burningFire
	elseif lastBurnSmolder == 0 and thisBurnSmolder == 1 then nextState = burningSmolder
	elseif lastBurnOff == 0 and thisBurnOff == 1 then         nextState = burningNot end
	
	-- Remote trigger
	local remTrig = (electrics.values['remoteTrig'] or 0)
	if remTrig ~= burningNoChange then
		if nextState == burningNoChange then
			nextState = remTrig
		end
		electrics.values['remoteTrig'] = burningNoChange
	end
	
	-- Save burn button states
	lastBurnOff = thisBurnOff
	lastBurnSmolder = thisBurnSmolder
	lastBurnFire = thisBurnFire
	lastBurnExplosion = thisBurnExplosion
	
	-- Bail
	return nextState
end

-- Graphics update function; Updates other stuff too according to dt
local function updateGFX(dt)
	
	-- Get possible next burn state based on buttons
	local burnButtonState = nextButtonBurnState()
	
	--Determine change in burn state
	if burnButtonState ~= burningNoChange then
		burnState = burnButtonState
		if burnState == burningExplosion then
			expTimeElap = 0
			if castLight ~= 0 then electrics.values['fireLight'] = 1 end
		elseif burnState == burningFire then
			if castLight ~= 0 then electrics.values['fireLight'] = 1 end
		elseif burnState == burningSmolder then
			electrics.values['fireLight'] = 0
		elseif burnState == burningNot then
			electrics.values['fireLight'] = 0
		end
	elseif burnState == burningExplosion then
		expTimeElap = expTimeElap + dt
		if expTimeElap >= explosionDuration then
			burnState = burningFire
		end
	end
	
	-- Do burn stuff
	if canFire == 1 then
		if burnTime >= burnInterval then
			burnTime = 0
			-- obj:addParticleByNodesRelative(node1,	node2,	speed,					type,			width,					count)
			if burnState == burningSmolder then
				obj:addParticleByNodesRelative(nFbtm,	nFtop,	smolderSmokeSpeed,		43,				smolderSmokeWidth,		smolderSmokeCount) -- Smoke
			elseif burnState == burningFire then
				obj:addParticleByNodesRelative(nFbtm,	nFtop,	fireFlameSpeed,			29,				fireFlameWidth,			fireFlameCount) -- Fire
				obj:addParticleByNodesRelative(nFbtm,	nFtop,	fireSmokeSpeed,			fireSmokeThick,	fireSmokeWidth,			fireSmokeCount) -- Smoke
			elseif burnState == burningExplosion then
				obj:addParticleByNodesRelative(nFbtm,	nFtop,	explosionFlameSpeed,	31,				explosionFlameWidth,	explosionFlameCount) -- Fire
				obj:addParticleByNodesRelative(nFbtm,	nFtop,	explosionSmokeSpeed,	32,				explosionSmokeWidth,	explosionSmokeCount) -- Smoke
			end
		end
		burnTime = burnTime + dt
	end
	
	-- Debug message
	if ugfxCalled == 0 then
		ugfxCalled = 1
		log("I", "HB9K Fire", "updateGFX called")
	end
end

-- Reset what we need to
local function reset()
	log("I", "HB9K Fire", "reset called")
	burnState = 0
	lastBurnOff = 0
	lastBurnSmolder = 0
	lastBurnFire = 0
	lastBurnExplosion = 0
	expTimeElap = 0
	electrics.values['fireLight'] = 0
end

-- Retrieve variables
local function getVars()

	--Smolder smoke
	smolderSmokeSpeed = (v.data.variables["$hb9k_smolderSmokeSpeed"].val or 0)
	smolderSmokeWidth = (v.data.variables["$hb9k_smolderSmokeWidth"].val or 0)
	smolderSmokeCount = (v.data.variables["$hb9k_smolderSmokeCount"].val or 0)
	
	-- Fire flames
	fireFlameSpeed = (v.data.variables["$hb9k_fireFlameSpeed"].val or 0)
	fireFlameWidth = (v.data.variables["$hb9k_fireFlameWidth"].val or 0)
	fireFlameCount = (v.data.variables["$hb9k_fireFlameCount"].val or 0)
	
	--Fire smoke
	fireSmokeSpeed = (v.data.variables["$hb9k_fireSmokeSpeed"].val or 0)
	fireSmokeWidth = (v.data.variables["$hb9k_fireSmokeWidth"].val or 0)
	fireSmokeCount = (v.data.variables["$hb9k_fireSmokeCount"].val or 0)
	if (v.data.variables["$hb9k_fireSmokeThick"].val or 0) == 0 then
		fireSmokeThick = 43
		fireSmokeSpeed = fireSmokeSpeed * 2
		fireSmokeWidth = fireSmokeWidth * 15
	else
		fireSmokeThick = 52
	end
	
	--Explosion timing
	explosionDuration = (v.data.variables["$hb9k_explosionDuration"].val or 0) / 1000
	
	--Explosion flames
	explosionFlameSpeed = (v.data.variables["$hb9k_explosionFlameSpeed"].val or 0)
	explosionFlameWidth = (v.data.variables["$hb9k_explosionFlameWidth"].val or 0)
	explosionFlameCount = (v.data.variables["$hb9k_explosionFlameCount"].val or 0)
	
	--Explosion smoke
	explosionSmokeSpeed = (v.data.variables["$hb9k_explosionSmokeSpeed"].val or 0)
	explosionSmokeWidth = (v.data.variables["$hb9k_explosionSmokeWidth"].val or 0)
	explosionSmokeCount = (v.data.variables["$hb9k_explosionSmokeCount"].val or 0)
	
	--Light
	castLight = (v.data.variables["$hb9k_castLight"].val or 0)
	
	--Timing
	burnInterval = (v.data.variables["$hb9k_burnRate"].val or 0)
	if burnInterval == 0 then
		burnInterval = 0.0333
	else
		burnInterval = 1 / burnInterval
	end
end

-- Initialize everything
local function init(jbeamdata)
	log("I", "HB9K Fire", "init called")
	
	-- Get emitter nodes
	for k, node in pairs (v.data.nodes) do
		if node.name == "nbctr" then nFbtm = k elseif
		   node.name == "ntctr" then nFtop = k end
	end
	if nFbtm ~= nil and nFtop ~= nil then canFire = 1 end
	
	-- Get variables
	getVars()
end

-- Public interface
M.init = init
M.reset = reset
M.updateGFX = updateGFX

-- Return the module
return M
